Stan Math Library: stan/math/prim/scal/prob/inv_chi_square_ccdf_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

inv_chi_square_ccdf_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_INV_CHI_SQUARE_CCDF_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_INV_CHI_SQUARE_CCDF_LOG_HPP
3
4 #include <boost/random/chi_squared_distribution.hpp>
5 #include <boost/random/variate_generator.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
9 #include <stan/math/prim/scal/err/check_not_nan.hpp>
10 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/fun/multiply_log.hpp>
13 #include <stan/math/prim/scal/fun/value_of.hpp>
14 #include <stan/math/prim/scal/fun/gamma_q.hpp>
15 #include <stan/math/prim/scal/fun/digamma.hpp>
16 #include <stan/math/prim/scal/meta/length.hpp>
17 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
18 #include <stan/math/prim/scal/meta/VectorView.hpp>
19 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
20 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
21 #include <stan/math/prim/scal/meta/return_type.hpp>
22 #include <stan/math/prim/scal/fun/grad_reg_inc_gamma.hpp>
23 #include <stan/math/prim/scal/meta/include_summand.hpp>
24 #include <cmath>
25 #include <limits>
26
27 namespace stan {
28
29 namespace math {
30
31 template <typename T_y, typename T_dof>
32 typename return_type<T_y, T_dof>::type
33 inv_chi_square_ccdf_log(const T_y& y, const T_dof& nu) {
34 typedef typename stan::partials_return_type<T_y, T_dof>::type
35 T_partials_return;
36
37 // Size checks
38 if ( !( stan::length(y) && stan::length(nu) ) ) return 0.0;
39
40 // Error checks
41 static const char* function("stan::math::inv_chi_square_ccdf_log");
42
43 using stan::math::check_positive_finite;
44 using stan::math::check_not_nan;
45 using stan::math::check_consistent_sizes;
46 using stan::math::check_nonnegative;
47 using boost::math::tools::promote_args;
48 using stan::math::value_of;
49 using std::exp;
50
51 T_partials_return P(0.0);
52
53 check_positive_finite(function, "Degrees of freedom parameter", nu);
54 check_not_nan(function, "Random variable", y);
55 check_nonnegative(function, "Random variable", y);
56 check_consistent_sizes(function,
57 "Random variable", y,
58 "Degrees of freedom parameter", nu);
59
60 // Wrap arguments in vectors
61 VectorView<const T_y> y_vec(y);
62 VectorView<const T_dof> nu_vec(nu);
63 size_t N = max_size(y, nu);
64
65 OperandsAndPartials<T_y, T_dof> operands_and_partials(y, nu);
66
67 // Explicit return for extreme values
68 // The gradients are technically ill-defined, but treated as zero
69
70 for (size_t i = 0; i < stan::length(y); i++)
71 if (value_of(y_vec[i]) == 0)
72 return operands_and_partials.value(0.0);
73
74 // Compute ccdf_log and its gradients
75 using stan::math::gamma_q;
76 using stan::math::digamma;
77 using boost::math::tgamma;
78 using std::exp;
79 using std::pow;
80 using std::log;
81
82 // Cache a few expensive function calls if nu is a parameter
83 VectorBuilder<!is_constant_struct<T_dof>::value,
84 T_partials_return, T_dof> gamma_vec(stan::length(nu));
85 VectorBuilder<!is_constant_struct<T_dof>::value,
86 T_partials_return, T_dof> digamma_vec(stan::length(nu));
87
88 if (!is_constant_struct<T_dof>::value) {
89 for (size_t i = 0; i < stan::length(nu); i++) {
90 const T_partials_return nu_dbl = value_of(nu_vec[i]);
91 gamma_vec[i] = tgamma(0.5 * nu_dbl);
92 digamma_vec[i] = digamma(0.5 * nu_dbl);
93 }
94 }
95
96 // Compute vectorized ccdf_log and gradient
97 for (size_t n = 0; n < N; n++) {
98 // Explicit results for extreme values
99 // The gradients are technically ill-defined, but treated as zero
100 if (value_of(y_vec[n]) == std::numeric_limits<double>::infinity()) {
101 return operands_and_partials.value(stan::math::negative_infinity());
102 }
103
104 // Pull out values
105 const T_partials_return y_dbl = value_of(y_vec[n]);
106 const T_partials_return y_inv_dbl = 1.0 / y_dbl;
107 const T_partials_return nu_dbl = value_of(nu_vec[n]);
108
109 // Compute
110 const T_partials_return Pn = 1.0 - gamma_q(0.5 * nu_dbl, 0.5
111 * y_inv_dbl);
112
113 P += log(Pn);
114
115 if (!is_constant_struct<T_y>::value)
116 operands_and_partials.d_x1[n] -= 0.5 * y_inv_dbl * y_inv_dbl
117 * exp(-0.5*y_inv_dbl) * pow(0.5*y_inv_dbl, 0.5*nu_dbl-1)
118 / tgamma(0.5*nu_dbl) / Pn;
119 if (!is_constant_struct<T_dof>::value)
120 operands_and_partials.d_x2[n]
121 -= 0.5 * stan::math::grad_reg_inc_gamma(0.5 * nu_dbl,
122 0.5 * y_inv_dbl,
123 gamma_vec[n],
124 digamma_vec[n]) / Pn;
125 }
126
127 return operands_and_partials.value(P);
128 }
129 }
130 }
131 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::inv_chi_square_ccdf_log
return_type< T_y, T_dof >::type inv_chi_square_ccdf_log(const T_y &y, const T_dof &nu)
Definition: inv_chi_square_ccdf_log.hpp:33
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
digamma.hpp
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::grad_reg_inc_gamma
T grad_reg_inc_gamma(T a, T z, T g, T dig, T precision=1e-6)
Definition: grad_reg_inc_gamma.hpp:16
return_type.hpp
gamma_q.hpp
include_summand.hpp
multiply_log.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
VectorView.hpp
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
length.hpp
grad_reg_inc_gamma.hpp
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
stan::math::pow
fvar< T > pow(const fvar< T > &x1, const fvar< T > &x2)
Definition: pow.hpp:18
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
value_of.hpp
stan::math::tgamma
fvar< T > tgamma(const fvar< T > &x)
Definition: tgamma.hpp:15
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
stan::math::gamma_q
fvar< T > gamma_q(const fvar< T > &x1, const fvar< T > &x2)
Definition: gamma_q.hpp:15
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::negative_infinity
double negative_infinity()
Return negative infinity.
Definition: constants.hpp:132
stan::math::digamma
fvar< T > digamma(const fvar< T > &x)
Definition: digamma.hpp:16

[ Stan Home Page ] © 2011–2016, Stan Development Team.